import os
from subprocess import check_output

import jujuresources
from charmhelpers.core import hookenv
from charmhelpers.core import unitdata
from charmhelpers.contrib.bigdata import utils
from path import Path
from sparkutils import update_etc_hosts_in_place
from socket import gethostname



class Spark(object):

    def __init__(self, dist_config):
        self.dist_config = dist_config
        self.cpu_arch = check_output(['uname', '-p']).strip()

    def is_installed(self):
        return unitdata.kv().get('spark.installed')

    def install(self, force=False):
        if not force and self.is_installed():
            return
        mirror_url = hookenv.config()['resources_mirror']
        jujuresources.fetch('spark-%s' % self.cpu_arch, mirror_url=mirror_url)
        jujuresources.install('spark-%s' % self.cpu_arch,
                              destination=self.dist_config.path('spark'),
                              skip_top_level=True)
        self.install_demo()
        self.dist_config.add_users()
        self.dist_config.add_dirs()
        self.dist_config.add_packages()
        self.setup_spark_config()
        self.configure_spark()
        utils.initialize_kv_host()
        #utils.update_etc_hosts(utils.get_kv_hosts())
        update_etc_hosts_in_place(hookenv.unit_private_ip(), gethostname())
        utils.get_ssh_key("ubuntu")
        utils.disable_firewall()
        unitdata.kv().set('spark.installed', True)
        
    def install_demo(self):
        demo_source = 'scripts/demo.sh'
        demo_target = '/home/ubuntu/demo.sh'
        Path(demo_source).copy(demo_target)
        Path(demo_target).chmod(0o755)
        Path(demo_target).chown('ubuntu', 'ubuntu')
        
    def setup_spark_config(self):
        # copy default config into alternate dir
        conf_dir = self.dist_config.path('spark') / 'conf'
        self.dist_config.path('spark_conf').rmtree_p()
        conf_dir.copytree(self.dist_config.path('spark_conf'))
        unitdata.kv().set('spark_conf', self.dist_config.path('spark_conf'))
        conf_dir.rmtree_p()
        spark_env = self.dist_config.path('spark_conf') / 'spark-env.sh'
        if not spark_env.exists():
            (self.dist_config.path('spark_conf') / 'spark-env.sh.template').copy(spark_env)
        spark_site = self.dist_config.path('spark_conf') / 'spark-defaults.conf'
        if not spark_site.exists():
            (self.dist_config.path('spark_conf') / 'spark-defaults.conf.template').copy(spark_site)
        spark_log4j = self.dist_config.path('spark_conf') / 'log4j.properties'
        if not spark_log4j.exists():
            (self.dist_config.path('spark_conf') / 'log4j.properties.template').copy(spark_log4j)
        utils.re_edit_in_place(spark_log4j, {
                r'log4j.rootCategory=INFO, console': 'log4j.rootCategory=ERROR, console',
                })
        
    def configure_spark(self):
        from subprocess import call
        spark_bin = self.dist_config.path('spark') / 'bin'
        spark_home = self.dist_config.path('spark')
        cmd = "chown -R ubuntu:hadoop {}".format (spark_home)
        call(cmd.split())
        cmd = "chown -R ubuntu:hadoop {}".format (self.dist_config.path('spark_conf'))
        call(cmd.split())
        unitdata.kv().set("spark_home", spark_home)
        unitdata.kv().set("hadoop_conf", self.dist_config.path('hadoop_conf'))
        with utils.environment_edit_in_place('/etc/environment') as env:
            if spark_bin not in env['PATH']:
                env['PATH'] = ':'.join([env['PATH'], spark_bin])
            env['SPARK_HOME']=spark_home
            env['IPYTHON']="1"
            env['SPARK_CONF_DIR'] = self.dist_config.path('spark_conf')
            env['HADOOP_CONF_DIR'] = self.dist_config.path('hadoop_conf')
       
            
    def start(self):
        import subprocess
       
        
    def stop(self):
        import subprocess
        
    def cleanup(self):
        self.dist_config.remove_users()
        self.dist_config.remove_dirs()
